@@ -3,6 +3,8 @@ import re
 import sys
 import time
 
+__version__ = '0.10.7'
+
 # Color types, their color codes if it's style, their default reset codes, and
 # RegEx's for detecting their color type.
 COLOR_TYPES = {
@@ -49,8 +51,9 @@ COLOR_TYPES = {
 # The format of a palette color
 PALETTE_COLOR_RE = re.compile(r'\b([bf])\.([a-z0-9-_]+)\b')
 
-# Select Graphic Rendition sequence (any type)
-SGR_RE = re.compile(br'\x1b\[[0-9;]*m')
+# Select Graphic Rendition sequence (any type or only colors)
+SGR_RE = re.compile(br'\x1b\x5b[\x30-\x3f]*[\x20-\x2f]*\x6d')
+SGR_COLOR_RE = re.compile(br'\x1b\x5b[0-9;]*\x6d')
 
 
 class Color:
@@ -157,7 +160,7 @@ class Color:
 
         self._rgb = value
 
-        # Update the color if preset; it won't be during __init__
+        # Update the color if present; it won't be during __init__
         if hasattr(self, '_color'):
             self.color = self._color
 
@@ -171,6 +174,10 @@ class Color:
             source_color_code (bytes): Bytes to be split into individual colors.
             is_reset (bool): Consider all identified colors as resets.
         '''
+        # Includes non-color characters; don't touch it
+        if not SGR_COLOR_RE.search(source_color_code):
+            return [[source_color_code, False, None]]
+
         make_sgr = lambda code_id: b'\x1b[' + code_id + b'm'
         colors = []
         codes = source_color_code.lstrip(b'\x1b[').rstrip(b'm').split(b';')
@@ -348,7 +355,7 @@ class Palette:
 class Rule:
     '''A rule containing a regex and colors corresponding to the regex's groups.'''
 
-    # pylint: disable=import-outside-toplevel,too-many-arguments
+    # pylint: disable=import-outside-toplevel,too-many-arguments,too-many-instance-attributes
     def __init__(self,
                  regex,
                  color=None,
@@ -371,17 +378,14 @@ class Rule:
         if description is not None and not isinstance(description, str):
             raise TypeError('description must be a string')
 
-        if not isinstance(regex, str):
-            raise TypeError('regex must be a string')
-
         if not isinstance(exclusive, bool):
             raise TypeError('exclusive must be a boolean')
 
-        self.regex = regex
         self.colors = {}
         self.description = description
         self.exclusive = exclusive
-        self.pcre = pcre  # Compiles regex into self._regex
+        self.pcre = pcre
+        self.regex = regex
 
         if not isinstance(color, dict):
             color = {0: color}
@@ -410,11 +414,27 @@ class Rule:
 
         self._pcre = value
 
-        if value:
+        # Recompile the regex if present; it won't be during __init__
+        if hasattr(self, '_regex'):
+            self.regex = self._regex
+
+    @property
+    def regex(self):
+        '''The regex pattern.'''
+        return self._regex
+
+    @regex.setter
+    def regex(self, value):
+        if not isinstance(value, str):
+            raise TypeError('regex must be a string')
+
+        self._regex = value
+
+        if self.pcre:
             import chromaterm.pcre
-            self._regex = chromaterm.pcre.Pattern(self.regex.encode())
+            self._regex_object = chromaterm.pcre.Pattern(value.encode())
         else:
-            self._regex = re.compile(self.regex.encode())
+            self._regex_object = re.compile(value.encode())
 
     def get_matches(self, data):
         '''Returns a list of tuples, each containing a start index, an end index,
@@ -425,7 +445,7 @@ class Rule:
         '''
         matches = []
 
-        for match in self._regex.finditer(data):
+        for match in self._regex_object.finditer(data):
             for group in self.colors:
                 start, end = match.span(group)
 
@@ -455,11 +475,11 @@ class Rule:
             raise TypeError('group must be an integer or a string')
 
         if isinstance(group, str):
-            if not self._regex.groupindex.get(group):
+            if not self._regex_object.groupindex.get(group):
                 raise ValueError(f'named group {repr(group)} not in regex')
 
             # Resolve the named group to its index
-            group = self._regex.groupindex[group]
+            group = self._regex_object.groupindex[group]
 
         if color is None:
             self.colors.pop(group, None)
@@ -468,9 +488,9 @@ class Rule:
         if not isinstance(color, Color):
             raise TypeError('color must be a chromaterm.Color')
 
-        if group > self._regex.groups:
-            raise ValueError(f'regex has {self._regex.groups} group(s); '
-                             f'{group} is invalid')
+        if group > self._regex_object.groups:
+            raise ValueError(f'regex has {self._regex_object.groups} group(s);'
+                             f' {group} is invalid')
 
         self.colors[group] = color
 
