@@ -1,4 +1,5 @@
 '''The command line utility for ChromaTerm'''
+# pylint: disable=import-outside-toplevel
 import argparse
 import atexit
 import io
@@ -11,10 +12,7 @@ from ctypes.util import find_library
 
 import yaml
 
-from chromaterm import Color, Config, Palette, Rule
-
-# Some sections are rarely used, so avoid unnecessary imports to speed up launch
-# pylint: disable=import-outside-toplevel
+from chromaterm import Color, Config, Palette, Rule, __version__
 
 # Possible locations of the config file, without the extension. Most-specific
 # locations lead in the list.
@@ -24,9 +22,6 @@ CONFIG_LOCATIONS = [
     '/etc/chromaterm/chromaterm',
 ]
 
-# The frequency to check the child process' `cwd` and update our own
-CWD_UPDATE_INTERVAL = 1 / 4
-
 # Maximum chuck size per read
 READ_SIZE = 8192
 
@@ -91,6 +86,11 @@ def args_init(args=None):
                         help='use RGB colors (default: detect support, '
                         'fallback to xterm-256)')
 
+    parser.add_argument('-v',
+                        '--version',
+                        action='version',
+                        version=f'%(prog)s {__version__}')
+
     parser.add_argument('--pcre',
                         action='store_true',
                         help="(advanced) use PCRE2 instead of Python's RE"
@@ -98,7 +98,7 @@ def args_init(args=None):
 
     args = parser.parse_args(args=args)
 
-    # Detect rgb support
+    # Detect rgb support if not specified
     args.rgb = args.rgb or os.getenv('COLORTERM') in ('truecolor', '24bit')
 
     return args
@@ -276,21 +276,25 @@ def process_input(config, data_fd, forward_fd=None, max_wait=None):
 
     Args:
         config (chromaterm.Config): Used for highlighting the data.
-        data_fd (int): File descriptor to be read and highlighted.
-        forward_fd (int): File descriptor to forwarded into data_fd. This is used
-            in conjunction with run_program. None indicates forwarding is not
-            required.
+        data_fd (int): File descriptor or socket to be read and highlighted.
+        forward_fd (int): File descriptor or socket to be forwarded into data_fd.
         max_wait (float): The maximum time to wait with no data on either of the
             file descriptors. None will block until at least one ready to be read.
     '''
     # pylint: disable=too-many-branches
-    # Avoid BlockingIOError when output to non-blocking stdout is too fast
+    # #93: Avoid BlockingIOError when output to non-blocking stdout is too fast
     try:
-        os.set_blocking(sys.stdout.fileno(), True)
+        getattr(os, 'set_blocking', lambda *_: None)(sys.stdout.fileno(), True)
     except io.UnsupportedOperation:
-        # In case stdout is replaced with StringIO
         pass
 
+    if isinstance(data_fd, int):
+        forward = lambda: os.write(data_fd, os.read(forward_fd, READ_SIZE))
+        read = lambda: os.read(data_fd, READ_SIZE)
+    else:
+        forward = lambda: data_fd.sendall(forward_fd.recv(READ_SIZE))
+        read = lambda: data_fd.recv(READ_SIZE)
+
     fds = [data_fd] if forward_fd is None else [data_fd, forward_fd]
     buffer = b''
 
@@ -300,7 +304,7 @@ def process_input(config, data_fd, forward_fd=None, max_wait=None):
         # There's some data to forward to the spawned program
         if forward_fd in ready_fds:
             try:
-                os.write(data_fd, os.read(forward_fd, READ_SIZE))
+                forward()
             except OSError:
                 # Spawned program or stdin closed; don't forward anymore
                 fds.remove(forward_fd)
@@ -308,7 +312,7 @@ def process_input(config, data_fd, forward_fd=None, max_wait=None):
         # Data to be highlighted was received
         if data_fd in ready_fds:
             try:
-                data_read = os.read(data_fd, READ_SIZE)
+                data_read = read()
             except OSError:
                 data_read = b''
 
@@ -341,12 +345,12 @@ def process_input(config, data_fd, forward_fd=None, max_wait=None):
                 buffer = data + separator
             # A single character indicates keyboard typing; don't highlight
             # Account for backspaces added by some shells, like zsh
-            elif 0 < len(data_read) < 2 + data_read.count(b'\b') * 2:
+            elif 0 < len(buffer) < 2 + data_read.count(b'\b') * 2:
                 sys.stdout.buffer.write(data + separator)
                 buffer = b''
             # There's more data; fetch it before highlighting, if buffer isn't full
             elif data_read and len(buffer) <= READ_SIZE \
-                    and read_ready(*fds, timeout=wait_duration):
+                    and data_fd in read_ready(*fds, timeout=wait_duration):
                 buffer = data + separator
             else:
                 sys.stdout.buffer.write(config.highlight(data) + separator)
@@ -387,8 +391,8 @@ def read_ready(*fds, timeout=None):
     return [] if not fds else select.select(fds, [], [], timeout)[0]
 
 
-def reload_chromaterm_instances():
-    '''Reloads other ChromaTerm CLI instances by sending them `signal.SIGUSR1`.
+def signal_chromaterm_instances(sig):
+    '''Sends `sig` signal to all other ChromaTerm CLI instances.
 
     Returns:
         The number of processes reloaded.
@@ -406,7 +410,7 @@ def reload_chromaterm_instances():
         try:
             # Only compare the first two arguments (Python and script paths)
             if process.cmdline()[:2] == current_process.cmdline()[:2]:
-                os.kill(process.pid, signal.SIGUSR1)
+                os.kill(process.pid, sig)
                 count += 1
         except (psutil.AccessDenied, psutil.NoSuchProcess):
             pass
@@ -414,80 +418,6 @@ def reload_chromaterm_instances():
     return count
 
 
-def run_program(program_args):
-    '''Spawns a program in a pty fork to emulate a controlling terminal.
-
-    Args:
-        program_args (list): A list of program arguments. The first argument is
-            the program name/location.
-
-    Returns:
-        A file descriptor (int) of the mater end of the pty fork.
-    '''
-    import fcntl
-    import pty
-    import termios
-    import threading
-    import time
-    import tty
-
-    try:
-        # Set to raw as the pty will be handling any processing; restore at exit
-        attributes = termios.tcgetattr(sys.stdin.fileno())
-        atexit.register(termios.tcsetattr, sys.stdin.fileno(), termios.TCSANOW,
-                        attributes)
-        tty.setraw(sys.stdin.fileno(), termios.TCSANOW)
-    except termios.error:
-        attributes = None
-
-    # openpty, login_tty, then fork
-    pid, master_fd = pty.fork()
-
-    if pid == 0:
-        # Update the slave's pty (now on standard fds) attributes
-        if attributes:
-            termios.tcsetattr(sys.stdin.fileno(), termios.TCSANOW, attributes)
-
-        try:
-            os.execvp(program_args[0], program_args)
-        except OSError as exception:
-            eprint(f'{exception.strerror.lower()}: {program_args[0]}')
-
-        # exec replaces the fork's process; only hit on exception
-        sys.exit(1)
-    else:
-        # Update the slave's window size as the master is capturing the signals
-        def window_resize_handler(*_):
-            size = fcntl.ioctl(sys.stdin.fileno(), termios.TIOCGWINSZ, '0000')
-            fcntl.ioctl(master_fd, termios.TIOCSWINSZ, size)
-
-        if sys.stdin.isatty():
-            signal.signal(signal.SIGWINCH, window_resize_handler)
-            window_resize_handler()
-
-        # Some terminals update their titles based on `cwd` (see #94)
-        def update_cwd():  # pragma: no cover
-            # Covered by `test_main_cwd_tracking` but not detected by coverage
-            import psutil
-
-            try:
-                child_process = psutil.Process(pid)
-            except (psutil.AccessDenied, psutil.NoSuchProcess):
-                return
-
-            while True:
-                try:
-                    time.sleep(CWD_UPDATE_INTERVAL)
-                    os.chdir(child_process.cwd())
-                except (OSError, psutil.AccessDenied, psutil.NoSuchProcess):
-                    pass
-
-        # Daemonized to exit immediately with ChromaTerm
-        threading.Thread(target=update_cwd, daemon=True).start()
-
-        return master_fd
-
-
 def split_buffer(buffer):
     '''Returns a tuple of tuples in the format of (data, separator). data should
     be highlighted while separator should be printed, unchanged, after data.
@@ -518,10 +448,11 @@ def main(args=None, max_wait=None, write_default=True):
     Returns:
         A string indicating status/error. Otherwise, returns None.
     '''
+    # pylint: disable=expression-not-assigned
     args = args_init(args)
 
     if args.reload:
-        return f'Processes reloaded: {reload_chromaterm_instances()}'
+        return f'Processes reloaded: {signal_chromaterm_instances(signal.SIGUSR1)}'
 
     # Config file wasn't overridden; use default file
     if not args.config:
@@ -538,17 +469,18 @@ def main(args=None, max_wait=None, write_default=True):
     if args.benchmark:
         atexit.register(config.print_benchmark_results)
 
+    import chromaterm.platform.unix as platform
+
     if args.program:
-        # ChromaTerm is spawning the program in a controlling terminal; stdin is
-        # being forwarded to the program
-        data_fd = run_program([args.program] + args.arguments)
-        forward_fd = sys.stdin.fileno()
+        # ChromaTerm is spawning the program in a pty; stdin is forwarded
+        data_fd = platform.run_program([args.program] + args.arguments)
+        forward_fd = platform.get_stdin()
     else:
         # Data is being piped into ChromaTerm's stdin; no forwarding needed
-        data_fd = sys.stdin.fileno()
+        data_fd = platform.get_stdin()
         forward_fd = None
 
-    # Create the signal handler to trigger reloading the config
+    # Signal handler to trigger reloading the config
     def reload_config_handler(*_):
         config_data = read_file(args.config)
 
@@ -568,6 +500,9 @@ def main(args=None, max_wait=None, write_default=True):
     except BrokenPipeError:
         # Surpress the implicit flush that Python runs on exit
         sys.stdout.flush = lambda: None
+    finally:
+        # Close data_fd to signal to the child process that we're done
+        os.close(data_fd) if isinstance(data_fd, int) else data_fd.close()
 
     return os.wait()[1] >> 8 if args.program else 0
 
